//+------------------------------------------------------------------+
//|                                                   Stochastic.mq4 |
//|                      Copyright  2004, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
// http://forum.mql4.com/17501 

#property copyright "Copyright  2004, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net/"
 
 
#property indicator_separate_window
#property indicator_minimum 0
#property indicator_maximum 100
#property indicator_buffers 2
#property indicator_color1 LightSeaGreen
#property indicator_color2 Red
//---- input parameters
extern int KPeriod=21;
extern int DPeriod=9;
extern int Slowing=9;
extern int level2=80;
extern int level1=20;
extern bool ShowSignals = true;
extern bool Signals_SW  = false;

extern bool alertSound    = true;
extern bool alertBox      = true;
extern bool alertEmail    = false;

//---- buffers
double MainBuffer[];
double SignalBuffer[];
double HighesBuffer[];
double LowesBuffer[];
//----
int draw_begin1=0;
int draw_begin2=0;
string short_name;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   
//---- 2 additional buffers are used for counting.
   IndicatorBuffers(4);
   SetIndexBuffer(2, HighesBuffer);
   SetIndexBuffer(3, LowesBuffer);
//---- indicator lines
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0, MainBuffer);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1, SignalBuffer);
//---- name for DataWindow and indicator subwindow label
   short_name="Sto("+KPeriod+","+DPeriod+","+Slowing+")";
   IndicatorShortName(short_name);
   SetIndexLabel(0,short_name);
   SetIndexLabel(1,"Signal");
//----
   draw_begin1=KPeriod+Slowing;
   draw_begin2=draw_begin1+DPeriod;
   SetIndexDrawBegin(0,draw_begin1);
   SetIndexDrawBegin(1,draw_begin2);
//----
   return(0);
  }



int deinit()
   {
		     for(int i = 0; i < ObjectsTotal(); i++)
		       {
			        if(StringFind( ObjectName(i), Symbol()+"_", 0 ) == 0)
			          {
				           ObjectDelete(ObjectName(i));
				           i--;
			          }
		       }

return(0);
   } 



//+------------------------------------------------------------------+
//| Stochastic oscillator                                            |
//+------------------------------------------------------------------+
int start()
  {
   int    i,k;
   int    counted_bars=IndicatorCounted();
   double price;
//----
   if(Bars<=draw_begin2) return(0);
//---- initial zero
   if(counted_bars<1)
     {
      for(i=1;i<=draw_begin1;i++) MainBuffer[Bars-i]=0;
      for(i=1;i<=draw_begin2;i++) SignalBuffer[Bars-i]=0;
     }
//---- minimums counting
   i=Bars-KPeriod;
   if(counted_bars>KPeriod) i=Bars-counted_bars-1;
   while(i>=0)
     {
      double min=1000000;
      k=i+KPeriod-1;
      while(k>=i)
        {
         price=Low[k];
         if(min>price) min=price;
         k--;
        }
      LowesBuffer[i]=min;
      i--;
     }
//---- maximums counting
   i=Bars-KPeriod;
   if(counted_bars>KPeriod) i=Bars-counted_bars-1;
   while(i>=0)
     {
      double max=-1000000;
      k=i+KPeriod-1;
      while(k>=i)
        {
         price=High[k];
         if(max<price) max=price;
         k--;
        }
      HighesBuffer[i]=max;
      i--;
     }
//---- %K line
   i=Bars-draw_begin1;
   if(counted_bars>draw_begin1) i=Bars-counted_bars-1;
   while(i>=0)
     {
      double sumlow=0.0;
      double sumhigh=0.0;
      for(k=(i+Slowing-1);k>=i;k--)
        {
         sumlow+=Close[k]-LowesBuffer[k];
         sumhigh+=HighesBuffer[k]-LowesBuffer[k];
        }
      if(sumhigh==0.0) MainBuffer[i]=100.0;
      else MainBuffer[i]=sumlow/sumhigh*100;
      i--;
     }
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   int limit=Bars-counted_bars;
//---- signal line is simple movimg average
   for(i=0; i<limit; i++)
      SignalBuffer[i]=iMAOnArray(MainBuffer,Bars,DPeriod,0,MODE_SMA,i);

//----
   
   bool play_sound=false;
   string subject="";

   int window =  WindowFind(short_name);
   for(i=0; i<limit; i++){
      double k1 = MainBuffer[i];
      double k2 = MainBuffer[i+1];
   
      double d1 = SignalBuffer[i];
      double d2 = SignalBuffer[i+1];
      
      // bull signal
      string bull_signal = Symbol()+"_"+iTime(NULL,0,i);
      if(k2<level1 && d2<level1)
         if(k2<d2 && k1> d1)    
         if(ObjectFind(bull_signal)==-1)
         {         

   if(ShowSignals &&!Signals_SW)
            ObjectCreate(bull_signal , OBJ_ARROW, 0, Time[i], (Low[i+1]-5*Point));
   if(ShowSignals && Signals_SW)          
            ObjectCreate(bull_signal,OBJ_ARROW,window,iTime(NULL,0,i),k1+5);      
            ObjectSet   (bull_signal,OBJPROP_ARROWCODE,241);
            ObjectSet   (bull_signal,OBJPROP_COLOR,Tomato);
            play_sound=true;
            subject = " BUY signal; M"+Period();
            
         }
      //bear signal
      string bear_signal = Symbol()+"_"+iTime(NULL,0,i);
      if(k2>level2 && d2>level2)
         if(k2>d2 && k1<d1)       
         if(ObjectFind(bear_signal)==-1)   
         {
   if(ShowSignals &&!Signals_SW)
            ObjectCreate(bear_signal , OBJ_ARROW, 0, Time[i], (High[i+1]+5*Point));
   if(ShowSignals && Signals_SW)          
            ObjectCreate(bear_signal,OBJ_ARROW,window,iTime(NULL,0,i),k1-5);                  
            ObjectSet   (bear_signal,OBJPROP_ARROWCODE,242);
            ObjectSet   (bear_signal,OBJPROP_COLOR,CornflowerBlue);
            play_sound=true;
            subject = " SELL signal; M"+Period();
         }
   }
   if(play_sound) {
         if(alertSound) PlaySound("tick.wav");   
         if(alertBox)   Alert("StoARR: ",Symbol(),subject);
         if(alertEmail) SendMail("StoARR: EA_ALERT "+Symbol(),subject);
   
   }  

   return(0);
  }
//+------------------------------------------------------------------+